using Nemerle.Collections;
using Nemerle.Compiler;
using Nemerle.Compiler.Typedtree;
using Nemerle.Macros;
using Oyster.Units;

namespace Oyster.Units.Macros
{
	using CT = CompileTimeUtil;

	/// Multiplies two units. Inserts usual * if operands are of wrong type
	macro @*(lexpr, rexpr)
	{
		CT.MulDivMacroBody(
			ImplicitCTX(),
			lexpr,
			rexpr,
			"*",
			<[ $lexpr * $rexpr ]>,
			_ + _,
			_ * _,
			<[ double.op_Multiply($lexpr.Value, $rexpr.Value) ]>)
	}

	/// Divides two units. Inserts usual / if operands are of wrong type
	macro @/(lexpr, rexpr)
	{
		CT.MulDivMacroBody(
			ImplicitCTX(),
			lexpr,
			rexpr,
			"/",
			<[ $lexpr / $rexpr ]>,
			_ - _,
			_ / _,
			<[ double.op_Division($lexpr.Value, $rexpr.Value) ]>)
	}

	/// Sums two units. Inserts usual + if operands are of wrong type.
	/// Throws compiler error if unit dimensions are not equal
	macro @+(lexpr, rexpr)
	{
		CT.AddSubMacroBody(
			ImplicitCTX(),
			lexpr,
			rexpr,
			"+",
			<[ $lexpr + $rexpr ]>,
			fun(arg2Expr) { <[ double.op_Addition($lexpr.Value, $arg2Expr) ]> })
	}

	/// Subtracts two units. Inserts usual - if operands are of wrong type.
	/// Throws compiler error if unit dimensions are not equal
	macro @-(lexpr, rexpr)
	{
		CT.AddSubMacroBody(
			ImplicitCTX(),
			lexpr,
			rexpr,
			"-",
			<[ $lexpr - $rexpr ]>,
			fun(arg2Expr) { <[ double.op_Subtraction($lexpr.Value, $arg2Expr) ]> })
	}
}
